const PATTERN = /Translation$/;
function findEnclosingTranslatedComputedProperty(t, path) {
    if (t.isFunctionExpression(path.node)) {
        let parentPath = path.parentPath;
        if (!t.isCallExpression(parentPath.node)) {
            return null;
        }
        let grandparentPath = parentPath.parentPath;
        if (!t.isObjectProperty(grandparentPath.node)) {
            return null;
        }
        return grandparentPath;
    }
    if (t.isFunctionDeclaration(path.node) || t.isArrowFunctionExpression(path.node)) {
        return null;
    }
    if (path.parentPath) {
        return findEnclosingTranslatedComputedProperty(t, path.parentPath);
    }
    return null;
}
class TranslatedComputedProperty {
    constructor(objectProperty) {
        this.objectProperty = objectProperty;
        this.returns = [];
    }
    addReturnIfMatching(t, returnStatementPath) {
        let returnStatement = returnStatementPath.node;
        if (!returnStatement.argument ||
            (!t.isStringLiteral(returnStatement.argument) &&
                !t.isTemplateLiteral(returnStatement.argument))) {
            return;
        }
        let translatedComputedPropertyPath = findEnclosingTranslatedComputedProperty(t, returnStatementPath);
        if (!translatedComputedPropertyPath) {
            return;
        }
        if (translatedComputedPropertyPath.node !== this.objectProperty) {
            return;
        }
        this.returns.push(returnStatementPath);
    }
    rewrite(t) {
        if (this.returns.length === 0) {
            return false;
        }
        if (this.objectProperty.computed || !t.isIdentifier(this.objectProperty.key)) {
            return false;
        }
        this.objectProperty.key.name = this.objectProperty.key.name.replace(PATTERN, '');
        for (let returnStatementPath of this.returns) {
            returnStatementPath.node.argument = t.callExpression(t.identifier('t'), [returnStatementPath.node.argument]);
        }
        return true;
    }
    static matches(t, objectProperty) {
        let key = objectProperty.key;
        let value = objectProperty.value;
        if (!t.isIdentifier(key)) {
            return false;
        }
        if (!t.isCallExpression(value)) {
            return false;
        }
        let lastArgument = value.arguments[value.arguments.length - 1];
        if (!t.isFunctionExpression(lastArgument)) {
            return false;
        }
        return PATTERN.test(key.name);
    }
}
export default function rewriteTranslationSuffixedComputedProperties(babel) {
    let t = babel.types;
    let enclosingFunctions = [];
    let enclosingFunction = null;
    let translatedComputedPropertyStack = [];
    return {
        visitor: {
            ObjectProperty: {
                enter(path) {
                    if (TranslatedComputedProperty.matches(t, path.node)) {
                        translatedComputedPropertyStack.push(new TranslatedComputedProperty(path.node));
                    }
                },
                exit(path) {
                    if (TranslatedComputedProperty.matches(t, path.node)) {
                        let translatedComputedProperty = translatedComputedPropertyStack.pop();
                        if (translatedComputedProperty) {
                            translatedComputedProperty.rewrite(t);
                        }
                    }
                }
            },
            ReturnStatement(path) {
                let translatedComputedProperty = translatedComputedPropertyStack[translatedComputedPropertyStack.length - 1];
                if (translatedComputedProperty) {
                    translatedComputedProperty.addReturnIfMatching(t, path);
                }
            }
        }
    };
}
